
<span class="type" title="{{ type }}">{{ type }}{% if is_iterable or is_dict %}:{{ object|length }}{% endif %}</span>

{% if intermediate %}
  <code class="intermediate">{{ intermediate }}</code>
{% endif %}

<span class="braces">{{ braces.0 }}</span>

<a
  class="arrow-toggle {{ collapsable.content.class }}"
  title="[Ctrl+click] Expand all children"
  data-toggle="collapse"
  data-target=".{{ unique }}{{ root_count }}"
  data-dd-type="type"
  data-object-depth="{{ depth|add:1 }}"
  aria-label="Close"
  aria-expanded="{{ collapsable.content.aria }}"
>
  <span class="unique" data-highlight-unique="{{ unique }}">{{ unique }}{{ root_count }}</span>
  <span id="arrow-{{ unique }}{{ root_count }}" class="arrow arrow-{{ unique }}{{ root_count }}">
    {{ collapsable.content.arrow }}
  </span>
</a>

<div
    class="dd-wrapper collapse {{ unique }}{{ root_count }} {{ collapsable.content.show }}"
    data-unique="{{ unique }}{{ root_count }}"
  >
  <ul class="attribute-list">
    {% if include_attributes %}
      {% include "django_dump_die/partials/_attributes.html" %}
    {% endif %}
  </ul>
  <ul class="attribute-list">
    {% if include_functions %}
      {% include "django_dump_die/partials/_functions.html" %}
    {% endif %}
  </ul>
</div>

<span class="braces">{{ braces.1 }}</span>
