{% load dump_die %}


<a
  class="arrow-toggle {{ collapsable.attribute.class }}"
  title="[Ctrl+click] Expand all children"
  {% if collapsable.attribute.always_show == 'false' %}
    data-toggle="collapse"
  {% endif %}
  data-target=".{{ unique }}{{ root_count }}-attributes"
  data-dd-type="attr"
  aria-label="Open/Close"
  aria-expanded="{{ collapsable.attribute.aria }}"
>
  <span class="section_name">Attributes</span>
  <span id="arrow-{{ unique }}{{ root_count }}-attributes" class="arrow">
    {{ collapsable.attribute.arrow }}
  </span>
</a>

<div
  class="li-wrapper collapse {{ unique }}{{ root_count }}-attributes {{ collapsable.attribute.show }}"
  data-unique-attributes="{{ unique }}{{ root_count }}-attributes"
>
  {% for attr, value, access_modifier, css_class, title in attributes %}
    <li>
      {% with depth|add:'1' as new_depth %}
        {% if access_modifier %}
          <span class="access-modifier">{{ access_modifier }}</span>
        {% endif %}
        {% if attr %}
          <span class="{{ css_class }}" title="{{ title }}">{{ attr }}</span>:
        {% endif %}
        {% if intermediate %}
          {% dump_object value root_obj skip 0 new_depth root_index_start root_index_end original_obj True %}
        {% elif is_iterable %}
          {% dump_object value root_obj skip forloop.counter0 new_depth root_index_start root_index_end original_obj False %}
        {% else %}
          {% dump_object value root_obj skip 0 new_depth root_index_start root_index_end original_obj False %}
        {% endif %}
      {% endwith %}
    </li>
  {% empty %}
    <span class="empty" title="No Attributes">No Attributes</span>
  {% endfor %}
</div>
