<div class="dump-toolbar">
  <div><h1>Django DumpDie</h1></div>
  <div>
    <div class="dropdown-container">
      <p id="expand-all" class="button">Expand All</p>
      <div class="dropdown-group">
        <p id="expand-1st-lvl" class="button">Expand <br>First Level</p>
        <p id="expand-2nd-lvl" class="button">Expand <br>Second Level</p>
        {% if attrs_enabled and funcs_enabled %}
          <p id="expand-types" class="button">Expand <br>Content</p>
          <p id="expand-attrs" class="button">Expand <br>Attributes</p>
          <p id="expand-funcs" class="button">Expand <br>Functions</p>
        {% endif %}
      </div>
    </div>
    <div class="dropdown-container">
      <p id="collapse-all" class="button">Collapse All</p>
      <div class="dropdown-group">
        <p id="collapse-1st-lvl" class="button">Collapse <br>First Level</p>
        <p id="collapse-2nd-lvl" class="button">Collapse <br>Second Level</p>
        {% if attrs_enabled and funcs_enabled %}
          <p id="collapse-types" class="button">Collapse <br>Content</p>
          <p id="collapse-attrs" class="button">Collapse <br>Attributes</p>
          <p id="collapse-funcs" class="button">Collapse <br>Functions</p>
        {% endif %}
      </div>
    </div>
  </div>
</div>
<div class="static-padding"></div>
