
<span class="type" title="{{ type }}">{{ type }}</span>

{% if intermediate %}
  <code class="datetime">{{ intermediate }}</code>
{% endif %}

<span class="braces">{{ braces.0 }}</span>

<a title="Already output or skipped object">
  <span class="unique" data-highlight-unique="{{ unique }}">{{ unique }}{{ root_count }}</span>
</a>

<span class="braces">{{ braces.1 }}</span>
